/*
*
*   Podprogramy pre pracu s USB pre virtualny seriovy port
*
*/

// Hlavickove subory
#include "board.h"
#include "usb_com.h"

// Definicie pouzitych typov
typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned int   uint;

// Definicia funkcie, ktora vracia minimum z dvoch cisel
#define MIN(a, b) (((a) < (b)) ? (a) : (b))

// Popisovac zariadenia
const char devDescriptor[] = {
	0x12,   // dlzka popisovaca
	0x01,   // typ popisovaca
	0x10,   // bcdUSBL
	0x01,   //
	0x02,   // kod triedy CDC
	0x00,   // kod podtriedy CDC
	0x00,   // kod protokolu CDC
	0x08,   // maximalna velkost paketu
	0xEB,   // kod vyrobcu (0x03EB - ATMEL)
	0x03,   //
	0x25,   // kod produktu (0x6125 - volny kod od Atmelu)
	0x61,   //
	0x10,   // bcdDeviceL
	0x01,   //
	0x00,   // vyrobca    // 0x01
	0x00,   // produkt
	0x00,   // seriove cislo
	0x01    // pocet konfiguracii
};

const char cfgDescriptor[] = {
	/* ============== Konfiguracia 1 =========== */
	/* popisovac konfiguracie 1 */
	0x09,   // dlzka
	0x02,   // typ popisovaca
	0x43,   // celkova dlzka (2 EP + Control)
	0x00,
	0x02,   // pocet rozhrani (interfejsov)
	0x01,   // CbConfigurationValue
	0x00,   // CiConfiguration
	0xC0,   // CbmAttributes 0xA0
	0x00,   // maximalny vykon

	/* Communication Class Interface Descriptor */
	0x09, // dlzka
	0x04, // typ popisovaca
	0x00, // cislo rozhrania
	0x00, // alternativne nastavenie
	0x01, // pocet endpointov
	0x02, // trieda rozhrania
	0x02, // podtrieda rozhrania
	0x00, // protokol rozhrania
	0x00, // rozhranie

	/* Header Functional Descriptor */
	0x05, // dlzka
	0x24, // typ popisovaca: CS_INTERFACE
	0x00, // podtyp popisovaca: Header Func Desc
	0x10, // bcdCDC:1.1
	0x01,

	/* ACM Functional Descriptor */
	0x04, // dlzka
	0x24, // typ popisovaca: CS_INTERFACE
	0x02, // podtyp popisovaca: ACM Func Desc
	0x00, // schopnosti (Capabilities)

	/* Union Functional Descriptor */
	0x05, // dlzka
	0x24, // typ popisovaca: CS_INTERFACE
	0x06, // podtyp popisovaca: Union Func Desc
	0x00, // rozhranie Master: Communication Class Interface
	0x01, // rozhranie Slave: Data Class Interface

	/* Call Management Functional Descriptor */
	0x05, // dlzka
	0x24, // typ popisovaca: CS_INTERFACE
	0x01, // podtyp popisovaca: Call Management Func Desc
	0x00, // schopnosti: D1 + D0
	0x01, // datove rozhranie: Data Class Interface 1

	/* popisovac edndpointu 1 */
	0x07,   // dlzka
	0x05,   // typ popisovaca
	0x83,   // adresa endpointu, Endpoint 03 - IN
	0x03,   // atributy - INTERRUPT (prenos s prerusenim)
	0x08,   // velkost paketu
	0x00,
	0xFF,   // interval

	/* Data Class Interface Descriptor */
	0x09, // dlzka
	0x04, // typ popisovaca
	0x01, // cislo rozhrania
	0x00, // alternativne nastavenie
	0x02, // pocet endpointov
	0x0A, // trieda rozhrania
	0x00, // podtrieda rozhrania
	0x00, // protokol rozhrania
	0x00, // rozhranie

	/* Prve alternativne nastavenie */
	/* popisovac endpointu 1 */
	0x07,   // dlzka
	0x05,   // typ popisovaca
	0x01,   // adresa endpointu, Endpoint 01 - OUT
	0x02,   // atributy - BULK (hromadny prenos)
	AT91C_EP_OUT_SIZE,   // maximalna velkost paketu
	0x00,
	0x00,   // interval

	/* popisovac endpointu 2 */
	0x07,   // dlzka
	0x05,   // typ popisovaca
	0x82,   // adresa endpointu, Endpoint 02 - IN
	0x02,   // atributy - BULK (hromadny prenos)
	AT91C_EP_IN_SIZE,   // maximalna velkost paketu
	0x00,
	0x00    // interval
};

/* Kody standardnych USB poziadaviek */
#define STD_GET_STATUS_ZERO           0x0080
#define STD_GET_STATUS_INTERFACE      0x0081
#define STD_GET_STATUS_ENDPOINT       0x0082

#define STD_CLEAR_FEATURE_ZERO        0x0100
#define STD_CLEAR_FEATURE_INTERFACE   0x0101
#define STD_CLEAR_FEATURE_ENDPOINT    0x0102

#define STD_SET_FEATURE_ZERO          0x0300
#define STD_SET_FEATURE_INTERFACE     0x0301
#define STD_SET_FEATURE_ENDPOINT      0x0302

#define STD_SET_ADDRESS               0x0500
#define STD_GET_DESCRIPTOR            0x0680
#define STD_SET_DESCRIPTOR            0x0700
#define STD_GET_CONFIGURATION         0x0880
#define STD_SET_CONFIGURATION         0x0900
#define STD_GET_INTERFACE             0x0A81
#define STD_SET_INTERFACE             0x0B01
#define STD_SYNCH_FRAME               0x0C82

/* Kody poziadaviek specificke pre danu CDC triedu */
#define GET_LINE_CODING               0x21A1
#define SET_LINE_CODING               0x2021
#define SET_CONTROL_LINE_STATE        0x2221

// Struktura pre definiciu vlastnosti serioveho portu
typedef struct {
	unsigned int dwDTERRate;
	char bCharFormat;
	char bParityType;
	char bDataBits;
} AT91S_CDC_LINE_CODING, *AT91PS_CDC_LINE_CODING;

// Vlastnosti serioveho portu
AT91S_CDC_LINE_CODING line = {
	115200, // rychlost
	0,      // 1 stop bit
	0,      // ziadna parita
	8};     // 8 datovych bitov


// Deklaracie pouzitych funkcii
static uchar AT91F_UDP_IsConfigured(AT91PS_CDC pCdc);
static uint AT91F_UDP_Read(AT91PS_CDC pCdc, char *pData, uint length);
static uint AT91F_UDP_Write(AT91PS_CDC pCdc, const char *pData, uint length);
static void AT91F_CDC_Enumerate(AT91PS_CDC pCdc);
static uchar AT91F_UDP_IsData(AT91PS_CDC pCdc);

//  *******************************************************
//  Iniciallizacia struktury CDC
//  *******************************************************
AT91PS_CDC AT91F_CDC_Open(AT91PS_CDC pCdc, AT91PS_UDP pUdp)
{
	pCdc->pUdp = pUdp;
	pCdc->currentConfiguration = 0;
	pCdc->currentConnection    = 0;
	pCdc->currentRcvBank       = AT91C_UDP_RX_DATA_BK0;
	pCdc->IsConfigured = AT91F_UDP_IsConfigured;
	pCdc->Write        = AT91F_UDP_Write;
	pCdc->Read         = AT91F_UDP_Read;
	pCdc->IsData       = AT91F_UDP_IsData;
	return pCdc;
}

//  *******************************************************
//  Kontrola, ci je zariadenie nakonfigurovane a obsluha enumeracie
//  *******************************************************
static uchar AT91F_UDP_IsConfigured(AT91PS_CDC pCdc)
{
	AT91PS_UDP pUDP = pCdc->pUdp;
	AT91_REG isr = pUDP->UDP_ISR;

	if (isr & AT91C_UDP_ENDBUSRES) {		// koniec resetu zbernice ?
		// zrusenie priznaku konca resetu
		pUDP->UDP_ICR = AT91C_UDP_ENDBUSRES;
		// reset vsetkych endpointov
		pUDP->UDP_RSTEP  = (unsigned int)-1;
		pUDP->UDP_RSTEP  = 0;
		// povolenie funkcie UDP
		pUDP->UDP_FADDR = AT91C_UDP_FEN;
		// konfiguracia endpointu 0 (jeho povolenie a nastavenie ako riadiaci endpoint)
		pUDP->UDP_CSR[0] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_CTRL);
	}
	else if (isr & AT91C_UDP_EPINT0) {		// prijem paketu cez endpoint 0 ?
		// vymazanie priznaku
		pUDP->UDP_ICR = AT91C_UDP_EPINT0;
		// enumeracia
		AT91F_CDC_Enumerate(pCdc);
	}
	return pCdc->currentConfiguration;
}

//  *******************************************************
//  Kontrola, ci boli prijate nejake udaje
//  *******************************************************
static uchar AT91F_UDP_IsData(AT91PS_CDC pCdc)
{
	AT91PS_UDP pUdp = pCdc->pUdp;
	uint currentReceiveBank = pCdc->currentRcvBank;

	// ak nie je UDP nakonfigurovane, koniec
	if ( !AT91F_UDP_IsConfigured(pCdc) )
	  return 0;
	// ak boli prijate udaje vrati 1
	if ( pUdp->UDP_CSR[AT91C_EP_OUT] & currentReceiveBank )
	  return 1;
	
	return 0;        // neboli prijate ziadne udaje
}

//  *******************************************************
//  Citanie udajov z endpoinu
//  *******************************************************
static uint AT91F_UDP_Read(AT91PS_CDC pCdc, char *pData, uint length)
{
	AT91PS_UDP pUdp = pCdc->pUdp;
	uint packetSize, nbBytesRcv = 0, currentReceiveBank = pCdc->currentRcvBank;

	// cyklus trva, kym vsetko neprecitame
	while (length) {
		// ak nie je UDP nakonfigurovane, koniec
		if ( !AT91F_UDP_IsConfigured(pCdc) )
			break;
		if ( pUdp->UDP_CSR[AT91C_EP_OUT] & currentReceiveBank ) {
			// precita sa nanajvys 'length' bajtov (porovnanie s aktualne prijatym poctom bajtov)
			packetSize = MIN(pUdp->UDP_CSR[AT91C_EP_OUT] >> 16, length);
			// zmensenie poctu ocakavanych udajov o tie prijate
			length -= packetSize;
			// ak je paket mensi nez 64 B (maximum), zrejme je to posledna cast udajov
			if (packetSize < AT91C_EP_OUT_SIZE)
				length = 0;
			// zapis udajov do FIFO
			while(packetSize--)
				pData[nbBytesRcv++] = pUdp->UDP_FDR[AT91C_EP_OUT];
			// nastavenie priznaku o precitani
			pUdp->UDP_CSR[AT91C_EP_OUT] &= ~(currentReceiveBank);
			// zmena banky 
			if (currentReceiveBank == AT91C_UDP_RX_DATA_BK0)
				currentReceiveBank = AT91C_UDP_RX_DATA_BK1;
			else
				currentReceiveBank = AT91C_UDP_RX_DATA_BK0;

		}
	}
	pCdc->currentRcvBank = currentReceiveBank;
	return nbBytesRcv;

}

//  *******************************************************
//  Vyslanie paketu cez endpoint 2
//  *******************************************************
static uint AT91F_UDP_Write(AT91PS_CDC pCdc, const char *pData, uint length)
{
	AT91PS_UDP pUdp = pCdc->pUdp;
	uint cpt = 0;

	// poslanie prveho paketu
	cpt = MIN(length, AT91C_EP_IN_SIZE);                     // maximalna dlzka paketu je 64 bajtov
	length -= cpt;
	while (cpt--) pUdp->UDP_FDR[AT91C_EP_IN] = *pData++;     // zapis udajov do FIFO
	pUdp->UDP_CSR[AT91C_EP_IN] |= AT91C_UDP_TXPKTRDY;        // paket je pripraveny

        // cyklus trva, kytm sa vsetko neposle
	while (length) {
		// naplnenie druhej banky
		cpt = MIN(length, AT91C_EP_IN_SIZE);                        // max 64 B
		length -= cpt;
		while (cpt--) pUdp->UDP_FDR[AT91C_EP_IN] = *pData++;        // zapis udajov
		// cakanie na vyslanie udajov z prvej banky
		while ( !(pUdp->UDP_CSR[AT91C_EP_IN] & AT91C_UDP_TXCOMP) )
			if ( !AT91F_UDP_IsConfigured(pCdc) ) return length;
                // vymazanie priznaku TXCOMP
		pUdp->UDP_CSR[AT91C_EP_IN] &= ~(AT91C_UDP_TXCOMP);
                // cakanie, kym UDP zrusi priznak TXCOMP
		while (pUdp->UDP_CSR[AT91C_EP_IN] & AT91C_UDP_TXCOMP);
		// dalsi paket je pripraveny
		pUdp->UDP_CSR[AT91C_EP_IN] |= AT91C_UDP_TXPKTRDY;
	}
	// cakanie na koniec prenosu
	while ( !(pUdp->UDP_CSR[AT91C_EP_IN] & AT91C_UDP_TXCOMP) )
		if ( !AT91F_UDP_IsConfigured(pCdc) ) return length;
	// vymazanie priznaku TXCOMP
	pUdp->UDP_CSR[AT91C_EP_IN] &= ~(AT91C_UDP_TXCOMP);
	// cakanie, kym UDP zrusi priznak TXCOMP
	while (pUdp->UDP_CSR[AT91C_EP_IN] & AT91C_UDP_TXCOMP);

	return length;
}

//  *******************************************************
//  Vyslanie udajov cez riadiaci endpoint 0
//  *******************************************************
unsigned int csrTab[100];
unsigned char csrIdx = 0;

static void AT91F_USB_SendData(AT91PS_UDP pUdp, const char *pData, uint length)
{
	uint cpt = 0;
	AT91_REG csr;

	// cyklus trva, kym sa vsetko neposle
	do {
		cpt = MIN(length, 8);                    // maximalne sa moze naraz vyslat 8 bajtov
		length -= cpt;

		while (cpt--)
			pUdp->UDP_FDR[0] = *pData++;     // zapis udajov do FIFO

		// ak treba, vynulovanie priznaku o prijati predchadzajucich udajov
		if (pUdp->UDP_CSR[0] & AT91C_UDP_TXCOMP) {
			pUdp->UDP_CSR[0] &= ~(AT91C_UDP_TXCOMP);
			while (pUdp->UDP_CSR[0] & AT91C_UDP_TXCOMP);
		}

		// nastavenie priznaku, ze paket je pripraveny
		pUdp->UDP_CSR[0] |= AT91C_UDP_TXPKTRDY;
		do {
			csr = pUdp->UDP_CSR[0];         // citanie stavu

			// citanie udajov bolo prerusene (status OUT)
			if (csr & AT91C_UDP_RX_DATA_BK0) {
				pUdp->UDP_CSR[0] &= ~(AT91C_UDP_RX_DATA_BK0);
				return;
			}
		} while ( !(csr & AT91C_UDP_TXCOMP) );  // cakanie na ukoncenie vyslania paketu

	} while (length);

	// ak treba, vynulovanie priznaku o prijati predchadzajucich udajov
	if (pUdp->UDP_CSR[0] & AT91C_UDP_TXCOMP) {
		pUdp->UDP_CSR[0] &= ~(AT91C_UDP_TXCOMP);
		while (pUdp->UDP_CSR[0] & AT91C_UDP_TXCOMP);
	}
}

//  *******************************************************
//  Poslanie paketu nulovej dlzky cez endpoint 0
//  *******************************************************
void AT91F_USB_SendZlp(AT91PS_UDP pUdp)
{
	// nastavenie priznaku, ze paket je pripraveny
	pUdp->UDP_CSR[0] |= AT91C_UDP_TXPKTRDY;
	// cakanie na potvrdenie o prijati paketu
	while ( !(pUdp->UDP_CSR[0] & AT91C_UDP_TXCOMP) );
	// vymazanie potvrdenia
	pUdp->UDP_CSR[0] &= ~(AT91C_UDP_TXCOMP);
	// cakanie, kym UDP nastavi sme chceli
	while (pUdp->UDP_CSR[0] & AT91C_UDP_TXCOMP);
}

//  *******************************************************
//  Vyslanie STALL paketu
//  *******************************************************
void AT91F_USB_SendStall(AT91PS_UDP pUdp)
{
	// vynutenie STALL paketu
	pUdp->UDP_CSR[0] |= AT91C_UDP_FORCESTALL;
	// cakanie na potvrdenie prijatia STALL paketu
	while ( !(pUdp->UDP_CSR[0] & AT91C_UDP_ISOERROR) );
	// zrusenie poziadavky na vynutenie STALL paketu a priznaku akceptovania STALL paketu
	pUdp->UDP_CSR[0] &= ~(AT91C_UDP_FORCESTALL | AT91C_UDP_ISOERROR);
	// cakanie, kym UDP nastavi sme chceli
	while (pUdp->UDP_CSR[0] & (AT91C_UDP_FORCESTALL | AT91C_UDP_ISOERROR));
}

//  *******************************************************
//  Enumeracia
//  *******************************************************
static void AT91F_CDC_Enumerate(AT91PS_CDC pCdc)
{
	AT91PS_UDP pUDP = pCdc->pUdp;
	uchar bmRequestType, bRequest;
	ushort wValue, wIndex, wLength, wStatus;

	// zaujimaju nas len SETUP pakety
	if ( !(pUDP->UDP_CSR[0] & AT91C_UDP_RXSETUP) )
		return;

	// nacitanie SETUP paketu - ma vzdy 8 bajtov
	bmRequestType = pUDP->UDP_FDR[0];
	bRequest      = pUDP->UDP_FDR[0];
	wValue        = (pUDP->UDP_FDR[0] & 0xFF);
	wValue       |= (pUDP->UDP_FDR[0] << 8);
	wIndex        = (pUDP->UDP_FDR[0] & 0xFF);
	wIndex       |= (pUDP->UDP_FDR[0] << 8);
	wLength       = (pUDP->UDP_FDR[0] & 0xFF);
	wLength      |= (pUDP->UDP_FDR[0] << 8);

	// kontrola smeru komunikacie (1 = smer IN - od zariadenia smerom k hostovi)
	if (bmRequestType & 0x80) {
		// nastavenie pozadovaneho smeru prenosu
		pUDP->UDP_CSR[0] |= AT91C_UDP_DIR;
		// cakanie, kym UDP periferia nastavi smer
		while ( !(pUDP->UDP_CSR[0] & AT91C_UDP_DIR) );
	}
	// oznamenie pre UDP, ze paket bol precitany z FIFO
	pUDP->UDP_CSR[0] &= ~AT91C_UDP_RXSETUP;
	// cakanie, kym UDP zrusi priznak prijatia paketu
	while ( (pUDP->UDP_CSR[0]  & AT91C_UDP_RXSETUP)  );

	// obsluha standardnych USB poziadaviek (tabulka 9-3, str.250 v usb_20.pdf)
	switch ((bRequest << 8) | bmRequestType) {
	case STD_GET_DESCRIPTOR:
		if (wValue == 0x100)       // poslanie popisovaca zariadenia (Device Descriptor)
			AT91F_USB_SendData(pUDP, devDescriptor, MIN(sizeof(devDescriptor), wLength));
		else if (wValue == 0x200)  // poslanie popisovaca konfiguracie (Configuration Descriptor)
			AT91F_USB_SendData(pUDP, cfgDescriptor, MIN(sizeof(cfgDescriptor), wLength));
		else
			AT91F_USB_SendStall(pUDP);        // STALL paket - ziadost o zrusenie poziadavky
		break;
	case STD_SET_ADDRESS:                                          // nastavenie adresy
		AT91F_USB_SendZlp(pUDP);                               // potvrdenie - paket nulovej dlzky
		pUDP->UDP_FADDR = (AT91C_UDP_FEN | wValue);            // ulozenie adresy
		// nastavenie stavu zariadenia (ak je adresa nenulova, nastavi sa stav 'adresovany')
		pUDP->UDP_GLBSTATE  = (wValue) ? AT91C_UDP_FADDEN : 0;
		break;
	case STD_SET_CONFIGURATION:                                    // nastavenie konfiguracie
		pCdc->currentConfiguration = wValue;
		AT91F_USB_SendZlp(pUDP);                               // potvrdenie - paket nulovej dlzky
		// nastavenie stavu zariadenia (ak je konfig. nenulova, nastavi sa stav 'konfigurovany')
		pUDP->UDP_GLBSTATE  = (wValue) ? AT91C_UDP_CONFG : AT91C_UDP_FADDEN;
		// nastavenie jednotlivych endpointov
		pUDP->UDP_CSR[1] = (wValue) ? (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_OUT) : 0;
		pUDP->UDP_CSR[2] = (wValue) ? (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_IN)  : 0;
		pUDP->UDP_CSR[3] = (wValue) ? (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_INT_IN)   : 0;
		break;
	case STD_GET_CONFIGURATION:                                    // citanie konfiguracie
		AT91F_USB_SendData(pUDP, (char *) &(pCdc->currentConfiguration), sizeof(pCdc->currentConfiguration));
		break;
	case STD_GET_STATUS_ZERO:                                      // citanie stavu zariadenia
		wStatus = 0;
		AT91F_USB_SendData(pUDP, (char *) &wStatus, sizeof(wStatus));
		break;
	case STD_GET_STATUS_INTERFACE:                                 // citanie stavu rozhrania
		wStatus = 0;
		AT91F_USB_SendData(pUDP, (char *) &wStatus, sizeof(wStatus));
		break;
	case STD_GET_STATUS_ENDPOINT:                                  // citanie stavu endpointu
		wStatus = 0;
		wIndex &= 0x0F;
		if ((pUDP->UDP_GLBSTATE & AT91C_UDP_CONFG) && (wIndex <= 3)) {
			wStatus = (pUDP->UDP_CSR[wIndex] & AT91C_UDP_EPEDS) ? 0 : 1;
			AT91F_USB_SendData(pUDP, (char *) &wStatus, sizeof(wStatus));
		}
		else if ((pUDP->UDP_GLBSTATE & AT91C_UDP_FADDEN) && (wIndex == 0)) {
			wStatus = (pUDP->UDP_CSR[wIndex] & AT91C_UDP_EPEDS) ? 0 : 1;
			AT91F_USB_SendData(pUDP, (char *) &wStatus, sizeof(wStatus));
		}
		else
			AT91F_USB_SendStall(pUDP);
		break;
	case STD_SET_FEATURE_ZERO:                                     // nastavenie vlastnosti zariadenia
		AT91F_USB_SendStall(pUDP);                             // nepodporovane - STALL paket
	    break;
	case STD_SET_FEATURE_INTERFACE:                                // nastavenie vlastnosti rozhrania
		AT91F_USB_SendZlp(pUDP);
		break;
	case STD_SET_FEATURE_ENDPOINT:                                 // nastavenie vlastnosti endpointu
		wIndex &= 0x0F;
		if ((wValue == 0) && wIndex && (wIndex <= 3)) {
			pUDP->UDP_CSR[wIndex] = 0;
			AT91F_USB_SendZlp(pUDP);
		}
		else
			AT91F_USB_SendStall(pUDP);
		break;
	case STD_CLEAR_FEATURE_ZERO:                                   // zrusenie vlastnosti zariadenia
		AT91F_USB_SendStall(pUDP);                             // nepodporovane - STALL paket
	    break;
	case STD_CLEAR_FEATURE_INTERFACE:                              // zrusenie vlastnosti rozhrania
		AT91F_USB_SendZlp(pUDP);
		break;
	case STD_CLEAR_FEATURE_ENDPOINT:                               // zrusenie vlastnosti endpointu
		wIndex &= 0x0F;
		if ((wValue == 0) && wIndex && (wIndex <= 3)) {
			if (wIndex == 1)
				pUDP->UDP_CSR[1] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_OUT);
			else if (wIndex == 2)
				pUDP->UDP_CSR[2] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_BULK_IN);
			else if (wIndex == 3)
				pUDP->UDP_CSR[3] = (AT91C_UDP_EPEDS | AT91C_UDP_EPTYPE_ISO_IN);
			AT91F_USB_SendZlp(pUDP);
		}
		else
			AT91F_USB_SendStall(pUDP);
		break;

	// obsluha specifickych poziadaviek pre danu triedu zariadeni
	case SET_LINE_CODING:                                          // nastavenie parametrov seriovej linky
                // cakanie na prijem udajov
		while ( !(pUDP->UDP_CSR[0] & AT91C_UDP_RX_DATA_BK0) );
                // oznamenie o precitani udajov (aj ked v skutocnosti neboli precitane)
		pUDP->UDP_CSR[0] &= ~(AT91C_UDP_RX_DATA_BK0);
                // vyslanie paketu nulovej dlzky
		AT91F_USB_SendZlp(pUDP);
		break;
	case GET_LINE_CODING:                                          // citanie parametrov seriovej linky
		AT91F_USB_SendData(pUDP, (char *) &line, MIN(sizeof(line), wLength));
		break;
	case SET_CONTROL_LINE_STATE:                                   // nastavenie stavu linky
		pCdc->currentConnection = wValue;
		AT91F_USB_SendZlp(pUDP);
		break;
	default:                                                       // na nezname poziadavky sa odpoveda
		AT91F_USB_SendStall(pUDP);                             // paketom STALL
	    break;
	}
}
